from django.http import HttpResponse
import siptracklib.errors

from siptrackweb.views import helpers
from siptrackweb.views import attribute
from siptrackweb.views import config
import category
from siptrackweb.forms import *

import siptracklib.template

def make_device_association_list(device):
    ret = []
    for assoc in device.listAssociations(include = ['device']):
        path = helpers.make_browsable_path(assoc,
                ['device category', 'device tree'],
                include_root = False)
        ent = {'obj': assoc, 'path': path, 'type': 'association'}
        ret.append(ent)
    for ref in device.listReferences(include = ['device']):
        path = helpers.make_browsable_path(ref,
                ['device category', 'device tree'],
                include_root = False)
        ent = {'obj': ref, 'path': path, 'type': 'reference'}
        ret.append(ent)
    ret.sort()
    return ret

def make_device_list_with_links(parent):
    ret = []
    for device in parent.listChildren(include = ['device']):
        ret.append(device)
        device.links = make_device_association_list(device)
    return ret

@helpers.authcheck
def index(request, view_oid):
    pm = helpers.PageManager(request, '')
    pm.section('device')
    view = pm.object_store.getOID(view_oid)
    device_tree = view.getChildByName('default', include = ['device tree'])
    return category.display(request, pm, device_tree)

@helpers.authcheck
def display(request, oid):
    pm = helpers.PageManager(request, '')
    device = pm.object_store.getOID(oid)
    if device.class_name == 'device':
        return display_device(request, pm, device)
    else:
        return category.display(request, pm, device)

@helpers.authcheck
def display_device(request, pm, device):
    pm.render_path = 'stweb/views/devices/display.html'
    pm.render_var['device'] = device
    pm.setVar('device_tree', device.getDeviceTree())
    pm.render_var['device_list'] = make_device_list_with_links(device)
    if device.attributes.get('reverse-device-sort-order', False):
        pm.render_var['device_list'].reverse()
    pm.render_var['password_list'] = device.listChildren(include = ['password'])
    pm.render_var['attribute_list'] = attribute.parse_attributes(device)
    pm.render_var['config_list'] = config.parse_config(device)
    if 'assigned network' in request.session:
        pm.render_var['assigned_network'] = request.session['assigned network']
        del request.session['assigned network']
    networks = device.listNetworks()
    hosts = [n for n in networks if n.isHost()]
    subnets = [n for n in networks if not n.isHost()]
    pm.render_var['network_host_list'] = hosts
    pm.render_var['network_subnet_list'] = subnets
    assoc_list = make_device_association_list(device)
    pm.render_var['device_association_list'] = assoc_list

    if pm.tagged_oid and pm.tagged_oid.oid != device.oid and \
            pm.tagged_oid.class_name == 'device':
        pm.render_var['valid_tag_target'] = True

    pm.path(device)

    return pm.render()

@helpers.authcheck
def delete(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    url = '/device/delete/post/%s/' % (device.oid)
    pm.addForm(DeleteForm(), url,
            'remove device', message = 'Removing device.')
    pm.path(device)

    return pm.render()

@helpers.authcheck
def delete_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    pm.path(device)
    url = '/device/delete/post/%s/' % (device.oid)
    pm.addForm(DeleteForm(request.POST), url,
            'remove device', message = 'Removing device.')
    if not pm.form.is_valid():
        return pm.render()

    parent_oid = device.parent.oid
    device.delete()

    return pm.redirect('device.display', (parent_oid,))

@helpers.authcheck
def add_template_select(request, parent_oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    parent = pm.setVar('parent', pm.object_store.getOID(parent_oid))
    pm.path(parent)

    templates = [template for template in \
            siptracklib.template.suggest_templates(parent, 'device template')
            if template.attributes.get('device_creation', False) and \
            not template.inheritance_only]
    url = '/device/add/template/set/%s/' % (parent_oid)
    pm.addForm(TemplateSelectForm(templates, True), url, 'create device')
    return pm.render()

@helpers.authcheck
def add_template_set(request, parent_oid):
    pm = helpers.PageManager(request, 'stweb/templates/apply_set.html')
    parent = pm.setVar('parent', pm.object_store.getOID(parent_oid))
    pm.path(parent)

    templates = [template for template in \
            siptracklib.template.suggest_templates(parent, 'device template')
            if template.attributes.get('device_creation', False)]
    form = TemplateSelectForm(templates, True, request.POST)
    if not form.is_valid():
        url = '/device/add/template/set/%s/' % (parent_oid)
        pm.addForm(form, url, 'create device')
        return pm.render()

    # No template chosen, just create an empty device.
    if form.cleaned_data['template'] == '-1':
        device = parent.add('device')
        return pm.redirect('device.display', (device.oid,))

    template = pm.object_store.getOID(form.cleaned_data['template'])
    pm.render_var['template'] = template
    url = '/device/add/template/post/%s/%s/' % (parent_oid, template.oid)
    form = TemplateSetForm(template)
    pm.addForm(form, url, 'create device')
    return pm.render()

@helpers.authcheck
def add_template_post(request, parent_oid, template_oid):
    pm = helpers.PageManager(request, 'stweb/templates/apply_set.html')
    parent = pm.setVar('parent', pm.object_store.getOID(parent_oid))
    pm.path(parent)

    template = pm.object_store.getOID(template_oid)
    pm.render_var['template'] = template

    url = '/device/add/template/post/%s/%s/' % (parent_oid, template.oid)
    form = TemplateSetForm(template, request.POST)
    pm.addForm(form, url, 'create device')
    if not pm.form.is_valid():
        return pm.error('')

    apply_rules = []
    for key, value in request.POST.iteritems():
        if not key.startswith('apply-argument'):
            continue
        _, _, rule_oid = key.split('-')
        apply_rules.append(rule_oid)

    arguments = {}
    for key in pm.form.cleaned_data:
        if not key.startswith('argument-'):
            continue
        prefix, rule_oid = key.split('-', 1)
        arguments[rule_oid] = [pm.form.cleaned_data[key]]

    skip_rules = []
    for rule in template.combinedRules():
        if rule.oid not in apply_rules:
            skip_rules.append(rule.oid)
        if rule.apply_arguments == 0 and rule.oid in arguments:
            del arguments[rule.oid]
        if rule.class_name == 'template rule password':
            arguments[rule.oid] = [helpers.generate_password()]
        if rule.class_name == 'template rule subdevice':
            arguments[rule.oid] = [{}, arguments[rule.oid][0]]

    device = parent.add('device')
    overwrite = True
    try:
        template.apply(device, arguments, overwrite, skip_rules)
    except siptracklib.errors.SiptrackError, e:
        device.delete()
        return pm.error(str(e))

    return pm.redirect('device.display', (device.oid,))

@helpers.authcheck
def disable(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    url = '/device/disable/post/%s/' % (device.oid)
    pm.addForm(EmptyForm(), url,
            'disable device', message = 'Disabling device.')
    pm.path(device)

    return pm.render()

@helpers.authcheck
def disable_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    pm.path(device)
    url = '/device/disable/post/%s/' % (device.oid)
    pm.addForm(EmptyForm(request.POST), url,
            'disable device', message = 'Disabling device.')
    if not pm.form.is_valid():
        return pm.render()

    device.attributes['disabled'] = True

    return pm.redirect('device.display', (oid,))

@helpers.authcheck
def enable(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    url = '/device/enable/post/%s/' % (device.oid)
    pm.addForm(EmptyForm(), url,
            'enable device', message = 'Enabling device.')
    pm.path(device)

    return pm.render()

@helpers.authcheck
def enable_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    pm.path(device)
    url = '/device/enable/post/%s/' % (device.oid)
    pm.addForm(EmptyForm(request.POST), url,
            'enable device', message = 'Enabling device.')
    if not pm.form.is_valid():
        return pm.render()

    disabled = device.attributes.getObject('disabled')
    if disabled is not None:
        disabled.delete()

    return pm.redirect('device.display', (oid,))

@helpers.authcheck
def recreate_template_select(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    pm.path(device)

    templates = [template for template in \
            siptracklib.template.suggest_templates(device, 'device template')
            if template.attributes.get('device_creation', False) and \
            not template.inheritance_only]
    url = '/device/recreate/template/set/%s/' % (oid)
    pm.addForm(TemplateSelectForm(templates, False), url, 're-create device')
    return pm.render()

@helpers.authcheck
def recreate_template_set(request, oid):
    pm = helpers.PageManager(request, 'stweb/templates/apply_set.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    pm.path(device)

    templates = [template for template in \
            siptracklib.template.suggest_templates(device, 'device template')
            if template.attributes.get('device_creation', False)]
    form = TemplateSelectForm(templates, False, request.POST)
    if not form.is_valid():
        url = '/device/recreate/template/set/%s/' % (oid)
        pm.addForm(form, url, 're-create device')
        return pm.render()

    template = pm.object_store.getOID(form.cleaned_data['template'])
    pm.render_var['template'] = template
    url = '/device/recreate/template/post/%s/%s/' % (oid, template.oid)
    form = TemplateSetForm(template)
    pm.addForm(form, url, 're-create device')
    return pm.render()

@helpers.authcheck
def recreate_template_post(request, oid, template_oid):
    pm = helpers.PageManager(request, 'stweb/templates/apply_set.html')
    device = pm.setVar('device', pm.object_store.getOID(oid))
    pm.path(device)

    template = pm.object_store.getOID(template_oid)
    pm.render_var['template'] = template

    url = '/device/recreate/template/post/%s/%s/' % (oid, template.oid)
    form = TemplateSetForm(template, request.POST)
    pm.addForm(form, url, 're-create device')
    if not pm.form.is_valid():
        return pm.error('')

    apply_rules = []
    for key, value in request.POST.iteritems():
        if not key.startswith('apply-argument'):
            continue
        _, _, rule_oid = key.split('-')
        apply_rules.append(rule_oid)

    arguments = {}
    for key in pm.form.cleaned_data:
        if not key.startswith('argument-'):
            continue
        prefix, rule_oid = key.split('-', 1)
        arguments[rule_oid] = [pm.form.cleaned_data[key]]

    skip_rules = []
    for rule in template.combinedRules():
        if rule.oid not in apply_rules:
            skip_rules.append(rule.oid)
        if rule.apply_arguments == 0 and rule.oid in arguments:
            del arguments[rule.oid]
        if rule.class_name == 'template rule password':
            arguments[rule.oid] = [helpers.generate_password()]
        if rule.class_name == 'template rule subdevice':
            arguments[rule.oid] = [{}, arguments[rule.oid][0]]

    overwrite = True
    try:
        template.apply(device, arguments, overwrite, skip_rules)
    except siptracklib.errors.SiptrackError, e:
        return pm.error(str(e))

    return pm.redirect('device.display', (device.oid,))

