from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import Context, loader
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from siptracklib.utils import object_by_attribute
import siptracklib.errors
import helpers
from siptrackweb.forms import *
from siptrackweb.views import attribute

def make_user_manager_list(view_tree):
    user_managers = []
    for um in view_tree.listChildren(include = ['user manager local',
        'user manager ldap']):
        data = {}
        data['obj'] = um
        if um is view_tree.user_manager:
            data['active'] = True
        else:
            data['active'] = False
        if um.class_name == 'user manager local':
            data['type'] = 'local'
        elif um.class_name == 'user manager ldap':
            data['type'] = 'ldap'
        user_managers.append(data)
    return user_managers

@helpers.authcheck
def index(request):
    pm = helpers.PageManager(request, 'stweb/users/index.html')
    pm.section('user')

    if request.session['administrator']:
        um_list = make_user_manager_list(pm.view_tree)
        pm.render_var['user_manager_list'] = um_list
        pm.render_var['session_list'] = pm.object_store.listServerSessions()
    else:
        user = pm.object_store.getSessionUser()
        pm.render_var['user'] = user
    pm.path(None)
    return pm.render()

@helpers.authcheck
def manager_display(request, oid):
    pm = helpers.PageManager(request, 'stweb/users/display_user_manager.html')
    pm.section('user')

    um = pm.object_store.getOID(oid)
    pm.render_var['user_manager'] = um
    pm.render_var['user_list'] = um.listChildren(include = ['user local',
        'user ldap'])
    pm.render_var['attribute_list'] = attribute.parse_attributes(um)
    pm.path(um)
    return pm.render()

@helpers.authcheck
def manager_updateusers(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    um = pm.object_store.getOID(oid)
    pm.addForm(EmptyForm(),
            '/user/manager/updateusers/post/%s/' % (oid),
            'update users', message = 'Update user list from ldap server.')
    pm.path(um)
    return pm.render()

@helpers.authcheck
def manager_updateusers_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    um = pm.object_store.getOID(oid)
    pm.path(um)
    pm.addForm(EmptyForm(request.POST),
            '/user/manager/updateusers/post/%s/' % (oid),
            'update users', message = 'Update user list from ldap server.')
    if not pm.form.is_valid():
        return pm.render()
    um.updateLDAPUsers(purge_missing_users = True)

    return pm.redirect('display.display', (oid,))

@helpers.authcheck
def manager_activate(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    um = pm.object_store.getOID(oid)
    pm.addForm(EmptyForm(),
            '/user/manager/activate/post/%s/' % (oid),
            'activate user manager', message = 'Activate user manager.')
    pm.path(um)
    return pm.render()

@helpers.authcheck
def manager_activate_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    um = pm.object_store.getOID(oid)
    pm.path(um)
    pm.addForm(EmptyForm(request.POST),
            '/user/manager/activate/post/%s/' % (oid),
            'activate user manager', message = 'Activate user manager.')
    if not pm.form.is_valid():
        return pm.render()
    pm.view_tree.user_manager = um

    return pm.redirect('display.display', (oid,))

@helpers.authcheck
def add(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    url = '/user/add/post/%s/' % (oid)
    pm.addForm(UserAddForm(), url, 'add user')
    pm.path(None)
    return pm.render()

@helpers.authcheck
def add_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    pm.path(None)

    url = '/user/add/post/%s/' % (oid)
    pm.addForm(UserAddForm(request.POST), url, 'add user')
    if not pm.form.is_valid():
        return pm.error()
    if pm.form.cleaned_data['password'] != pm.form.cleaned_data['validate']:
        return pm.error('passwords don\'t match')

    um = pm.object_store.getOID(oid)
    user_type = None
    if um.class_name == 'user manager local':
        user_type = 'user local'
    elif um.class_name == 'user manager ldap':
        user_type = 'user ldap'
    try:
        user = um.add(user_type,
                pm.form.cleaned_data['user_name'],
                pm.form.cleaned_data['password'],
                pm.form.cleaned_data['administrator'],
                pm.form.cleaned_data['write_access'])
    except siptracklib.errors.SiptrackError, e:
        return pm.error(str(e))
    user.attributes['real_name'] = pm.form.cleaned_data['real_name']
    user.attributes['description'] = pm.form.cleaned_data['description']

    return pm.redirect('display.display', (oid,))

@helpers.authcheck
def display(request, oid):
    pm = helpers.PageManager(request, 'stweb/users/display_user.html')

    user = pm.setVar('user', pm.object_store.getOID(oid))
    pm.render_var['attribute_list'] = attribute.parse_attributes(user)
    pm.render_var['subkey_list'] = user.listChildren(include = ['sub key'])
    pm.path(user)
    return pm.render()

@helpers.authcheck
def delete(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    user = pm.setVar('user', pm.object_store.getOID(oid))
    pm.addForm(DeleteForm(), '/user/delete/post/%s/' % (user.oid),
            'remove user', message = 'Removing user.')
    pm.path(user)
    return pm.render()

@helpers.authcheck
def delete_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    user = pm.object_store.getOID(oid)
    pm.addForm(DeleteForm(request.POST), '/user/delete/post/%s/' % (user.oid),
            'remove user', message = 'Removing user.')
    if not pm.form.is_valid():
        return pm.render()
    parent = user.parent
    user.delete()

    return pm.redirect('display.display', (parent.oid,))

@helpers.authcheck
def update(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    user = pm.setVar('user', pm.object_store.getOID(oid))
    form_data = {'user_name': user.username,
            'real_name': user.attributes.get('real_name', ''),
            'description': user.attributes.get('description', ''),
            'password': '',
            'validate': '',
            'administrator': user.administrator,
            'write_access': user.write_access}
    url = '/user/update/post/%s/' % (user.oid)
    if request.session['administrator']:
        pm.addForm(UserUpdateAdminForm(form_data), url, 'update user')
    else:
        pm.addForm(UserUpdateForm(form_data), url, 'update user')
    pm.path(user)
    return pm.render()

@helpers.authcheck
def update_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    user = pm.object_store.getOID(oid)
    url = '/user/update/post/%s/' % (user.oid)
    if request.session['administrator']:
        pm.addForm(UserUpdateAdminForm(request.POST), url, 'update user')
    else:
        pm.addForm(UserUpdateForm(request.POST), url, 'update user')
    if not pm.form.is_valid():
        return pm.error()
    if ((pm.form.cleaned_data['password'] != pm.form.cleaned_data['validate']) and
            (len(pm.form.cleaned_data['password']) > 0 or
                len(pm.form.cleaned_data['validate']) > 0)):
        return pm.error('passwords don\'t match')

    if pm.form.cleaned_data['user_name'] != user.username:
        user.setUsername(pm.form.cleaned_data['user_name'])
    if pm.form.cleaned_data['real_name'] != user.attributes.get('real_name', None):
        user.attributes['real_name'] = pm.form.cleaned_data['real_name']
    if pm.form.cleaned_data['description'] != user.attributes.get('description', None):
        user.attributes['description'] = pm.form.cleaned_data['description']
    if request.session['administrator']:
        if pm.form.cleaned_data['administrator'] != user.administrator:
            user.setAdministrator(pm.form.cleaned_data['administrator'])
        if pm.form.cleaned_data['write_access'] != user.write_access:
            user.setWriteAccess(pm.form.cleaned_data['write_access'])
    if len(pm.form.cleaned_data['password']) > 0:
        user.setPassword(pm.form.cleaned_data['password'])

    return pm.redirect('user.display', (oid,))

def _get_password_keys(view_tree):
    password_keys = []
    for view in view_tree.listChildren(include = ['view']):
        for password_key in view.listChildren(include = ['password key']):
            password_keys.append(password_key)
    return password_keys

@helpers.authcheck
def connectkey_selectkey(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    user = pm.object_store.getOID(oid)

    url = '/user/connectkey/post/%s/' % (oid)
    password_keys = _get_password_keys(pm.view_tree)
    require_user_password = True
    if pm.render_var['username'] == user.username:
        require_user_password = False
    form = UserConnectKeyForm(password_keys, require_user_password)
    pm.addForm(form, url, 'connect password key')
    pm.path(user)
    return pm.render()

@helpers.authcheck
def connectkey_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    user = pm.object_store.getOID(oid)

    url = '/user/connectkey/post/%s/' % (oid)
    password_keys = _get_password_keys(pm.view_tree)
    require_user_password = True
    if pm.render_var['username'] == user.username:
        require_user_password = False
    form = UserConnectKeyForm(password_keys, require_user_password, request.POST)
    pm.addForm(form, url, 'connect password key')
    pm.path(user)
    if not pm.form.is_valid():
        return pm.error()

    password_key = pm.object_store.getOID(pm.form.cleaned_data['passwordkey'])
    user_password = None
    if 'user_password' in pm.form.cleaned_data and \
            len(pm.form.cleaned_data['user_password']) > 0:
        user_password = pm.form.cleaned_data['user_password']
    password_key_key = None
    if len(pm.form.cleaned_data['password_key_key']) > 0:
        password_key_key = pm.form.cleaned_data['password_key_key']
    try:
        user.connectPasswordKey(password_key, user_password, password_key_key)
    except siptracklib.errors.SiptrackError, e:
        return pm.error(str(e))

    return pm.redirect('user.display', (oid,))

@helpers.authcheck
def subkey_delete(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    subkey = pm.setVar('subkey', pm.object_store.getOID(oid))
    user = subkey.parent
    pm.addForm(DeleteForm(), '/user/subkey/delete/post/%s/' % (oid),
            'remove subkey', message = 'Removing subkey.')
    pm.path(user)
    return pm.render()

@helpers.authcheck
def subkey_delete_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    subkey = pm.object_store.getOID(oid)
    user = subkey.parent
    pm.addForm(DeleteForm(request.POST), '/user/subkey/delete/post/%s/' % (oid),
            'remove subkey', message = 'Removing subkey.')
    if not pm.form.is_valid():
        return pm.render()
    subkey.delete()

    return pm.redirect('user.display', (user.oid,))

@helpers.authcheck
def session_kill(request, session_id):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    pm.addForm(DeleteForm(), '/user/session/kill/post/%s/' % (session_id),
            'kill session', message = 'Kill session.')
    pm.path(None)
    return pm.render()

@helpers.authcheck
def session_kill_post(request, session_id):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    pm.addForm(DeleteForm(request.POST), '/user/session/kill/post/%s/' % (session_id),
            'kill session', message = 'Kill session.')
    if not pm.form.is_valid():
        return pm.render()
    pm.object_store.killServerSession(session_id)

    return pm.redirect('user.index', ())

@helpers.authcheck
def manager_add(request, um_type):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    url = '/user/manager/add/post/%s/' % (um_type)
    if um_type == 'local':
        form = UserManagerLocalAddForm()
    elif um_type == 'ldap':
        form = UserManagerLDAPAddForm()
    else:
        raise Exception('no')
    pm.addForm(form, url, 'add user manager')
    pm.path(None)
    return pm.render()

@helpers.authcheck
def manager_add_post(request, um_type):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    pm.path(None)

    url = '/user/manager/add/post/%s/' % (um_type)
    if um_type == 'local':
        form = UserManagerLocalAddForm(request.POST)
    elif um_type == 'ldap':
        form = UserManagerLDAPAddForm(request.POST)
    else:
        raise Exception('no')
    pm.addForm(form, url, 'add user manager')
    if not pm.form.is_valid():
        return pm.error()

    try:
        if um_type == 'local':
            um = pm.view_tree.add('user manager local')
        elif um_type == 'ldap':
            valid_groups = []
            if len(pm.form.cleaned_data.get('valid_groups', '')) > 0:
                valid_groups = [pm.form.cleaned_data['valid_groups']]
            um = pm.view_tree.add('user manager ldap',
                pm.form.cleaned_data['connection_type'],
                pm.form.cleaned_data['server'],
                pm.form.cleaned_data['port'],
                pm.form.cleaned_data['base_dn'],
                valid_groups)
        else:
            raise Exception('no')
    except siptracklib.errors.SiptrackError, e:
        return pm.error(str(e))
    um.attributes['name'] = pm.form.cleaned_data['name']
    um.attributes['description'] = pm.form.cleaned_data.get('description', '')

    return pm.redirect('display.display', (um.oid,))

@helpers.authcheck
def manager_update(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    um = pm.object_store.getOID(oid)
    url = '/user/manager/update/post/%s/' % (oid)
    if um.class_name == 'user manager local':
        form_data = {'name': um.attributes.get('name', ''),
                'description': um.attributes.get('description', '')}
        pm.addForm(UserManagerLocalAddForm(form_data), url, 'update user manager')
    elif um.class_name == 'user manager ldap':
        valid_groups = ''
        if len(um.valid_groups) > 0:
            valid_groups = um.valid_groups[0]
        form_data = {'name': um.attributes.get('name', ''),
                'description': um.attributes.get('description', ''),
                'connection_type': um.connection_type,
                'server': um.server,
                'port': um.port,
                'base_dn': um.base_dn,
                'valid_groups': valid_groups,
                }
        pm.addForm(UserManagerLDAPAddForm(form_data), url, 'update user manager')
    else:
        raise Exception('no')
    pm.path(um)
    return pm.render()

@helpers.authcheck
def manager_update_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    um = pm.object_store.getOID(oid)
    url = '/user/manager/update/post/%s/' % (oid)
    if um.class_name == 'user manager local':
        pm.addForm(UserManagerLocalAddForm(request.POST), url, 'update user manager')
    elif um.class_name == 'user manager ldap':
        pm.addForm(UserManagerLDAPAddForm(request.POST), url, 'update user manager')
    else:
        raise Exception('no')
    if not pm.form.is_valid():
        return pm.error()
    if um.class_name == 'user manager ldap':
        valid_groups = []
        if len(pm.form.cleaned_data.get('valid_groups', '')) > 0:
            valid_groups = [pm.form.cleaned_data['valid_groups']]
        if pm.form.cleaned_data['connection_type'] != um.connection_type:
            um.setConnectionType(pm.form.cleaned_data['connection_type'])
        if pm.form.cleaned_data['server'] != um.server:
            um.setServer(pm.form.cleaned_data['server'])
        if pm.form.cleaned_data['port'] != um.port:
            um.setPort(pm.form.cleaned_data['port'])
        if pm.form.cleaned_data['base_dn'] != um.base_dn:
            um.setBaseDN(pm.form.cleaned_data['base_dn'])
        um.setValidGroups(valid_groups)

    if pm.form.cleaned_data['name'] != um.attributes.get('name', None):
        um.attributes['name'] = pm.form.cleaned_data['name']
    if pm.form.cleaned_data['description'] != um.attributes.get('description', None):
        um.attributes['description'] = pm.form.cleaned_data['description']

    return pm.redirect('display.display', (oid,))

@helpers.authcheck
def manager_delete(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')

    um = pm.object_store.getOID(oid)
    pm.addForm(DeleteForm(), '/user/manager/delete/post/%s/' % (oid),
            'remove user manager', message = 'Removing user manager.')
    pm.path(um)
    return pm.render()

@helpers.authcheck
def manager_delete_post(request, oid):
    pm = helpers.PageManager(request, 'stweb/generic_form.html')
    um = pm.object_store.getOID(oid)
    pm.addForm(DeleteForm(request.POST),
            '/user/manager/delete/post/%s/' % (oid),
            'remove user manager', message = 'Removing user manager.')
    if not pm.form.is_valid():
        return pm.render()
    try:
        um.delete()
    except siptracklib.errors.SiptrackError, e:
        return pm.error(str(e))

    return pm.redirect('user.index', ())

